# -*- coding: utf-8 -*-
## $Id: interfaces.py 16165 2020-07-14 14:19:29Z henrik $
##
## Copyright (C) 2011 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
"""Customized interfaces of the university application package.
"""

from zope import schema
from zope.component import getUtility
from zope.catalog.interfaces import ICatalog
from zc.sourcefactory.basic import BasicSourceFactory
from waeup.kofa.applicants.interfaces import (
    IApplicantBaseData,
    AppCatCertificateSource, CertificateSource)
from waeup.kofa.schoolgrades import ResultEntryField
from waeup.kofa.university.vocabularies import StudyModeSource
from waeup.kofa.interfaces import (
    SimpleKofaVocabulary, academic_sessions_vocab, validate_email,
    IKofaObject)
from waeup.kofa.schema import FormattedDate, TextLineChoice, PhoneNumber
from waeup.kofa.students.vocabularies import (
    nats_vocab, GenderSource, StudyLevelSource)
from waeup.kofa.applicants.interfaces import (
    contextual_reg_num_source, IApplicantBaseData, IApplicantRefereeReport)
from waeup.kofa.refereeentries import RefereeEntryField
from kofacustom.nigeria.applicants.interfaces import (
    LGASource, high_qual, high_grade, exam_types, DisabilitiesSource,
    programme_types_vocab, jambsubjects, validate_jamb_reg_number,
    INigeriaUGApplicant, INigeriaPGApplicant,
    INigeriaApplicantOnlinePayment,
    INigeriaUGApplicantEdit, INigeriaPGApplicantEdit,
    INigeriaApplicantUpdateByRegNo,
    IPUTMEApplicantEdit,
    )
from kofacustom.iuokada.interfaces import MessageFactory as _
from kofacustom.iuokada.payments.interfaces import ICustomOnlinePayment

sponsors_vocab = SimpleKofaVocabulary(
    (_('Bauchi Government'), 'bauchi'),
    (_('Company'), 'company'),
    (_('Federal Government of Nigeria'), 'federalgov'),
    #(_('Dangote Group'), 'dangote'),
    (_('Kano Government'), 'kano'),
    (_('Parent/Guardian'), 'parent'),
    (_('Rosula Organization'), 'rosula'),
    (_('Self'), 'self'),
    #(_('Social Impact Project'), 'social'),
    )

heard_about_types_vocab = SimpleKofaVocabulary(
    (_('Facebook/Twitter/Other Social Media'), 'socmedia'),
    (_('From a Friend'), 'friend'),
    (_('Newspaper Advertisement'), 'newspaper'),
    (_('Radio Advertisement'), 'radio'),
    (_('Television Advertisement'), 'television'),
    (_('SMS'), 'sms'),
    )

subtype_vocab = SimpleKofaVocabulary(
    (_('UTME'), 'utme'),
    (_('Direct Entry'), 'de'),
    (_('Inter Uni Transfer'), 'transfer'),
    )

rating_vocab = SimpleKofaVocabulary(
    (_('Excellent'), 'e'),
    (_('Very good'), 'vg'),
    (_('Good'), 'g'),
    (_('Slightly above average'), 'saa'),
    (_('Average'), 'a'),
    (_('Below average'), 'ba'),
    (_('Unable to assess'), 'unable'),
    )


overallpromise_vocab = SimpleKofaVocabulary(
    (_('Very good (highest 10%)'), 'vg'),
    (_('Above average (next 15%)'), 'aa'),
    (_('Average (middle 20%)'), 'a'),
    (_('Below average (middle 40%)'), 'ba'),
    )

DESTINATION_COST = {
    'local': ('Local (Nigeria)', 15000.0, 1),
    'overseas': ('Overseas', 25000.0, 2),
    }

class DestinationCostSource(BasicSourceFactory):
    """A source that delivers continents and shipment costs.
    """
    def getValues(self):
        sorted_items = sorted(DESTINATION_COST.items(),
                              key=lambda element: element[1][2])
        return [item[0] for item in sorted_items]

    def getTitle(self, value):
        return u"%s (₦ %s)" % (
            DESTINATION_COST[value][0],
            DESTINATION_COST[value][1])

class TranscriptCertificateSource(CertificateSource):
    """Include Department and Faculty in Title.
    """
    def getValues(self, context):
        catalog = getUtility(ICatalog, name='certificates_catalog')
        resultset = catalog.searchResults(code=(None, None))
        resultlist = sorted(resultset, key=lambda
            value: value.__parent__.__parent__.__parent__.code +
            value.__parent__.__parent__.code +
            value.code)
        return resultlist

    def getTitle(self, context, value):
        """
        """
        try: title = "%s / %s / %s (%s)" % (
            value.__parent__.__parent__.__parent__.title,
            value.__parent__.__parent__.title,
            value.title, value.code)
        except AttributeError:
            title = "NA / %s (%s)" % (value.title, value.code)
        return title

class ICustomUGApplicant(IApplicantBaseData):
    """An undergraduate applicant.

    This interface defines the least common multiple of all fields
    in ug application forms. In customized forms, fields can be excluded by
    adding them to the UG_OMIT* tuples.
    """

    subtype = schema.Choice(
        title = _(u'Application Subtype'),
        vocabulary = subtype_vocab,
        required = False,
        )
    disabilities = schema.Choice(
        title = _(u'Disability'),
        source = DisabilitiesSource(),
        required = False,
        )
    nationality = schema.Choice(
        source = nats_vocab,
        title = _(u'Nationality'),
        required = False,
        )
    lga = schema.Choice(
        source = LGASource(),
        title = _(u'State/LGA (Nigerians only)'),
        required = False,
        )
    sponsor = schema.Choice(
        title = _(u'Sponsor'),
        vocabulary = sponsors_vocab,
        required = False,
        )
    heard_about = schema.Choice(
        title = _(u'How did you hear about IUO?'),
        vocabulary = heard_about_types_vocab,
        required = False,
        )
    #perm_address = schema.Text(
    #    title = _(u'Permanent Address'),
    #    required = False,
    #    )
    parents_name = schema.TextLine(
        title = _(u'Full Name'),
        required = False,
        readonly = False,
        )
    parents_email = schema.ASCIILine(
        title = _(u'Email Address'),
        required = False,
        constraint=validate_email,
        )
    parents_phone = PhoneNumber(
        title = _(u'Phone'),
        required = False,
        )
    course1 = schema.Choice(
        title = _(u'1st Choice Course of Study'),
        source = AppCatCertificateSource(),
        required = True,
        )
    course2 = schema.Choice(
        title = _(u'2nd Choice Course of Study'),
        source = AppCatCertificateSource(),
        required = False,
        )
    programme_type = schema.Choice(
        title = _(u'Programme Type'),
        vocabulary = programme_types_vocab,
        required = False,
        )
    fst_sit_fname = schema.TextLine(
        title = _(u'Full Name'),
        required = False,
        readonly = False,
        )
    fst_sit_no = schema.TextLine(
        title = _(u'Exam Number'),
        required = False,
        readonly = False,
        )
    fst_sit_date = FormattedDate(
        title = _(u'Exam Date'),
        required = False,
        readonly = False,
        show_year = True,
        )
    fst_sit_type = schema.Choice(
        title = _(u'Exam Type'),
        required = False,
        readonly = False,
        vocabulary = exam_types,
        )
    fst_sit_results = schema.List(
        title = _(u'Exam Results'),
        value_type = ResultEntryField(),
        required = False,
        readonly = False,
        defaultFactory=list,
        )
    scd_sit_fname = schema.TextLine(
        title = _(u'Full Name'),
        required = False,
        readonly = False,
        )
    scd_sit_no = schema.TextLine(
        title = _(u'Exam Number'),
        required = False,
        readonly = False,
        )
    scd_sit_date = FormattedDate(
        title = _(u'Exam Date'),
        required = False,
        readonly = False,
        show_year = True,
        )
    scd_sit_type = schema.Choice(
        title = _(u'Exam Type'),
        required = False,
        readonly = False,
        vocabulary = exam_types,
        )
    scd_sit_results = schema.List(
        title = _(u'Exam Results'),
        value_type = ResultEntryField(),
        required = False,
        readonly = False,
        defaultFactory=list,
        )
    hq_type = schema.Choice(
        title = _(u'Qualification Obtained'),
        required = False,
        readonly = False,
        vocabulary = high_qual,
        )
    hq_matric_no = schema.TextLine(
        title = _(u'Former Matric Number'),
        required = False,
        readonly = False,
        )
    hq_degree = schema.Choice(
        title = _(u'Class of Degree'),
        required = False,
        readonly = False,
        vocabulary = high_grade,
        )
    hq_school = schema.TextLine(
        title = _(u'Institution Attended'),
        required = False,
        readonly = False,
        )
    hq_session = schema.TextLine(
        title = _(u'Years Attended'),
        required = False,
        readonly = False,
        )
    hq_disc = schema.TextLine(
        title = _(u'Discipline'),
        required = False,
        readonly = False,
        )
    #jamb_subjects = schema.Text(
    #    title = _(u'Subjects and Scores'),
    #    required = False,
    #    )
    jamb_subjects_list = schema.List(
        title = _(u'JAMB Subjects'),
        required = False,
        defaultFactory=list,
        value_type = schema.Choice(
            vocabulary = jambsubjects
            #source = JAMBSubjectSource(),
            ),
        )
    jamb_score = schema.Int(
        title = _(u'Total JAMB Score'),
        required = False,
        )
    #jamb_age = schema.Int(
    #    title = _(u'Age (provided by JAMB)'),
    #    required = False,
    #    )
    jamb_reg_number = schema.TextLine(
        title = _(u'JAMB Registration Number'),
        required = False,
        constraint=validate_jamb_reg_number,
        )
    notice = schema.Text(
        title = _(u'Notice'),
        required = False,
        )
    screening_venue = schema.TextLine(
        title = _(u'Screening Venue'),
        required = False,
        )
    screening_date = schema.TextLine(
        title = _(u'Screening Date'),
        required = False,
        )
    screening_score = schema.Int(
        title = _(u'Screening Score (%)'),
        required = False,
        )
    aggregate = schema.Int(
        title = _(u'Aggregate Score (%)'),
        description = _(u'(average of relative JAMB and PUTME scores)'),
        required = False,
        )
    result_uploaded = schema.Bool(
        title = _(u'Result uploaded'),
        default = False,
        required = False,
        )
    student_id = schema.TextLine(
        title = _(u'Student Id'),
        required = False,
        readonly = False,
        )
    course_admitted = schema.Choice(
        title = _(u'Admitted Course of Study'),
        source = CertificateSource(),
        required = False,
        )
    locked = schema.Bool(
        title = _(u'Form locked'),
        default = False,
        required = False,
        )

ICustomUGApplicant[
    'subtype'].order =  ICustomUGApplicant['lga'].order
ICustomUGApplicant[
    'locked'].order =  ICustomUGApplicant['suspended'].order
ICustomUGApplicant[
    'result_uploaded'].order =  ICustomUGApplicant['suspended'].order

class ICustomPGApplicant(INigeriaPGApplicant):
    """A postgraduate applicant.

    This interface defines the least common multiple of all fields
    in pg application forms. In customized forms, fields can be excluded by
    adding them to the PG_OMIT* tuples.
    """

    sponsor = schema.Choice(
        title = _(u'Sponsor'),
        vocabulary = sponsors_vocab,
        required = False,
        )

    heard_about = schema.Choice(
        title = _(u'How did you hear about IU?'),
        vocabulary = heard_about_types_vocab,
        required = False,
        )

    nysc_number = schema.Int(
        title = _(u'Nysc Number'),
        required = False,
        readonly = False,
        )

    referees = schema.List(
        title = _(u'Referees'),
        value_type = RefereeEntryField(),
        required = False,
        defaultFactory=list,
        )

ICustomPGApplicant[
    'referees'].order =  INigeriaPGApplicant['emp2_reason'].order
ICustomPGApplicant[
    'sponsor'].order =  ICustomPGApplicant['lga'].order
ICustomPGApplicant[
    'heard_about'].order =  ICustomPGApplicant['lga'].order
ICustomPGApplicant[
    'sponsor'].order =  ICustomPGApplicant['lga'].order
ICustomPGApplicant[
    'lga'].order =  ICustomPGApplicant['nationality'].order
ICustomPGApplicant[
    'nysc_number'].order =  ICustomPGApplicant['nysc_year'].order

class ITranscriptApplicant(IKofaObject):
    """A transcript applicant.
    """

    suspended = schema.Bool(
        title = _(u'Account suspended'),
        default = False,
        required = False,
        )

    locked = schema.Bool(
        title = _(u'Form locked'),
        default = False,
        required = False,
        )

    applicant_id = schema.TextLine(
        title = _(u'Transcript Application Id'),
        required = False,
        readonly = False,
        )

    matric_number = schema.TextLine(
        title = _(u'Matriculation Number'),
        readonly = False,
        required = True,
        )

    firstname = schema.TextLine(
        title = _(u'First Name'),
        required = True,
        )

    middlename = schema.TextLine(
        title = _(u'Middle Name'),
        required = False,
        )

    lastname = schema.TextLine(
        title = _(u'Last Name (Surname)'),
        required = True,
        )

    #date_of_birth = FormattedDate(
    #    title = _(u'Date of Birth'),
    #    required = False,
    #    #date_format = u'%d/%m/%Y', # Use grok-instance-wide default
    #    show_year = True,
    #    )

    sex = schema.Choice(
        title = _(u'Gender'),
        source = GenderSource(),
        required = True,
        )

    #nationality = schema.Choice(
    #    vocabulary = nats_vocab,
    #    title = _(u'Nationality'),
    #    required = False,
    #    )

    email = schema.ASCIILine(
        title = _(u'Email Address'),
        required = True,
        constraint=validate_email,
        )

    #phone = PhoneNumber(
    #    title = _(u'Phone'),
    #    description = u'',
    #    required = False,
    #    )

    #perm_address = schema.Text(
    #    title = _(u'Current Local Address'),
    #    required = False,
    #    readonly = False,
    #    )

    dispatch_address = schema.Text(
        title = _(u'Dispatch Addresses'),
        description = u'Addresses to which transcripts should be posted. '
                       'Addresses must involve same courier charges.',
        required = False,
        readonly = False,
        )

    charge = schema.Choice(
        title = _(u'Courier Charge'),
        source = DestinationCostSource(),
        required = True,
        )

    #entry_mode = schema.Choice(
    #    title = _(u'Entry Mode'),
    #    source = StudyModeSource(),
    #    required = False,
    #    readonly = False,
    #    )

    #entry_session = schema.Choice(
    #    title = _(u'Entry Session'),
    #    source = academic_sessions_vocab,
    #    required = False,
    #    readonly = False,
    #    )

    end_session = schema.Choice(
        title = _(u'Academic Session of Graduation'),
        source = academic_sessions_vocab,
        required = False,
        readonly = False,
        )

    course_studied = schema.Choice(
        title = _(u'Course of Study'),
        source = TranscriptCertificateSource(),
        description = u'Faculty / Department / Course',
        required = False,
        readonly = False,
        )

    #course_changed = schema.Choice(
    #    title = _(u'Change of Study Course'),
    #    description = u'If yes, select previous course of study.',
    #    source = CertificateSource(),
    #    readonly = False,
    #    required = False,
    #    )

    #change_level = schema.Choice(
    #    title = _(u'Change Level'),
    #    description = u'If yes, select level at which you changed course of study.',
    #    source = StudyLevelSource(),
    #    required = False,
    #    readonly = False,
    #    )

    no_copies = schema.Choice(
        title = _(u'Number of Copies'),
        description = u'Must correspond with the number of dispatch addresses above.',
        values=[1, 2, 3, 4],
        required = False,
        readonly = False,
        default = 1,
        )

class ICustomApplicant(ICustomUGApplicant, ICustomPGApplicant,
    ITranscriptApplicant):
    """An interface for both types of applicants.

    Attention: The ICustomPGApplicant field seetings will be overwritten
    by ICustomPGApplicant field settings. If a field is defined
    in both interfaces zope.schema validates only against the
    constraints in ICustomUGApplicant. This does not affect the forms
    since they are build on either ICustomUGApplicant or ICustomPGApplicant.
    """

    def writeLogMessage(view, comment):
        """Adds an INFO message to the log file
        """

    def createStudent():
        """Create a student object from applicant data
        and copy applicant object.
        """

class ICustomUGApplicantEdit(ICustomUGApplicant):
    """An undergraduate applicant interface for edit forms.

    Here we can repeat the fields from base data and set the
    `required` and `readonly` attributes to True to further restrict
    the data access. Or we can allow only certain certificates to be
    selected by choosing the appropriate source.

    We cannot omit fields here. This has to be done in the
    respective form page.
    """

    phone = PhoneNumber(
        title = _(u'Phone'),
        description = u'',
        required = True,
        )
    email = schema.ASCIILine(
        title = _(u'Email Address'),
        required = True,
        constraint=validate_email,
        )
    date_of_birth = FormattedDate(
        title = _(u'Date of Birth'),
        required = True,
        show_year = True,
        )

ICustomUGApplicantEdit[
    'date_of_birth'].order = ICustomUGApplicant['date_of_birth'].order
ICustomUGApplicantEdit[
    'email'].order = ICustomUGApplicant['email'].order
ICustomUGApplicantEdit[
    'phone'].order =  ICustomUGApplicantEdit['email'].order

class ICustomPGApplicantEdit(ICustomPGApplicant):
    """A postgraduate applicant interface for editing.

    Here we can repeat the fields from base data and set the
    `required` and `readonly` attributes to True to further restrict
    the data access. Or we can allow only certain certificates to be
    selected by choosing the appropriate source.

    We cannot omit fields here. This has to be done in the
    respective form page.
    """

    phone = PhoneNumber(
        title = _(u'Phone'),
        description = u'',
        required = True,
        )
    email = schema.ASCIILine(
        title = _(u'Email Address'),
        required = True,
        constraint=validate_email,
        )
    date_of_birth = FormattedDate(
        title = _(u'Date of Birth'),
        required = True,
        show_year = True,
        )

ICustomPGApplicantEdit[
    'date_of_birth'].order = ICustomPGApplicant['date_of_birth'].order
ICustomPGApplicantEdit[
    'email'].order = ICustomPGApplicant['email'].order
ICustomPGApplicantEdit[
    'phone'].order =  ICustomPGApplicantEdit['email'].order

class ICustomApplicantOnlinePayment(INigeriaApplicantOnlinePayment):
    """An applicant payment via payment gateways.

    """

class IPUTMEApplicantEdit(IPUTMEApplicantEdit):
    """An undergraduate applicant interface for editing.

    Here we can repeat the fields from base data and set the
    `required` and `readonly` attributes to True to further restrict
    the data access. Or we can allow only certain certificates to be
    selected by choosing the appropriate source.

    We cannot omit fields here. This has to be done in the
    respective form page.
    """

class ICustomApplicantUpdateByRegNo(INigeriaApplicantUpdateByRegNo):
    """Representation of an applicant.

    Skip regular reg_number validation if reg_number is used for finding
    the applicant object.
    """

class ICustomApplicantRefereeReport(IApplicantRefereeReport):
    """A referee report.
    """

    duration = schema.Text(
        title = _(u'How long and in what capacity have you known the candidate?'),
        required = False,
        )

    itellectual = schema.Choice(
        title = _(u'Intellectual Capacity'),
        required = False,
        readonly = False,
        vocabulary = rating_vocab,
        )

    persistent = schema.Choice(
        title = _(u'Capacity for Persistent and Independent Academic Study'),
        required = False,
        readonly = False,
        vocabulary = rating_vocab,
        )

    imaginative = schema.Choice(
        title = _(u'Ability for Imaginative Thought'),
        required = False,
        readonly = False,
        vocabulary = rating_vocab,
        )

    productive = schema.Choice(
        title = _(u'Promise of Productive Scholarship'),
        required = False,
        readonly = False,
        vocabulary = rating_vocab,
        )

    previous = schema.Choice(
        title = _(u'Quality of Previous Work'),
        required = False,
        readonly = False,
        vocabulary = rating_vocab,
        )

    expression = schema.Choice(
        title = _(u'Oral and Written Expression in English'),
        required = False,
        readonly = False,
        vocabulary = rating_vocab,
        )

    personality = schema.Text(
        title = _(u'Please comment on the candidate\'s personality '
            'with particular reference to his/her moral character, emotional '
            'and physical stabilty'),
        required = False,
        )

    promise = schema.Choice(
        title = _(u'Candidate\'s overall promise'),
        required = False,
        readonly = False,
        vocabulary = overallpromise_vocab,
        )

    report = schema.Text(
        title = _(u'Any other relevant information which would help '
            'in determining the candidate\'s suitability?'),
        required = False,
        )

    objection = schema.Text(
        title = _(u'Have you any objection to the contents of this '
            'evaluation being disclosed to any award-given body if '
            'the need arises?'),
        required = False,
        )